# -*- encoding: utf-8 -*-

import time
import platform
import json
import subprocess

from . import yy_wifi_helper

class YYUbuntuWIFIHelper(yy_wifi_helper.YYOSWIFIHelper):
    def __init__(self):
        pass

    def __del__(self):
        pass

    def disableEventHandler(self):
        pass

    def enableEventHandler(self, handler:yy_wifi_helper.YYWIFIHelper, debug:bool = False ):
        pass
    
    def getConnectedAPSSID(self, targetInterface: str | None = None) -> (bool, str | None, str | None):
        output = None

        checkInterface = self.getInterface()
        target = None
        if targetInterface == None:
            target = checkInterface['default']
        for iface in checkInterface['list']:
            if targetInterface == iface:
                target = targetInterface

        if target == None:
            return (False, f"interface not found: {targetInterface}, default: {checkInterface['default']}")

        cli = subprocess.Popen(['nmcli', 'device', 'show', target],
                stdin =subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                bufsize=0)
        cli.stdin.close()

        item = {}
        for line in cli.stdout:
            keyValue = line.split(":", 2)
            if len(keyValue) == 2:
                key = keyValue[0].strip()
                value = keyValue[1].strip()
                item[key] = value
        if 'GENERAL.CONNECTION' in item and item['GENERAL.CONNECTION'] != '--':
            output = item['GENERAL.CONNECTION']

        return (True, output)

    def disconnect(self, targetInterface:str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> (bool, str):
        timeCost = {
            'total': time.time(),
        }

        checkInterface = self.getInterface()
        foundInterface = None
        if targetInterface == None:
            foundInterface = checkInterface['default']
        for iface in checkInterface['list']:
            if targetInterface == iface:
                foundInterface = targetInterface

        if foundInterface == None:
            return (False, f"interface not found: {targetInterface}")

        cliParams = ['nmcli', 'device', 'disconnect', foundInterface]

        cli = subprocess.Popen(cliParams,
                stdin =subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                bufsize=0)
        cli.stdin.close()

        result = False
        cmdOutput = []
        for line in cli.stdout:
            cmdOutput.append(line)
            if line.find('successfully disconnected'):
                result = True

        timeCost['total'] = time.time() - timeCost['total']

        cmdOutput = "\n".join(cmdOutput)
        if result == False:
            cmdOutput = f"stdout:\n{cmdOutput}\n stderr:\n{cli.stderr.read()}\n"
        return (result, f"info: {timeCost}, cmd: {cmdOutput}")

    def connectToAP(self, targetSSID:str , targetPassword: str | None = None, targetSecurity: yy_wifi_helper.YYWIFISecurityMode | None = None, findSSIDBeforeConnect:bool = False, targetInterface: str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> (bool, str):
        timeCost = {
            'total': 0.0,
            'scan': 0.0,
            'connect': 0.0,
            'connected': 0.0,
        }
        timeCost['total'] = time.time()

        checkInterface = self.getInterface()
        foundInterface = None
        if targetInterface == None:
            foundInterface = checkInterface['default']
        for iface in checkInterface['list']:
            if targetInterface == iface:
                foundInterface = targetInterface

        if foundInterface == None:
            return (False, f"interface not found: {targetInterface}")

        result = False
                
        timeCost['connected'] = time.time()

        cliParams = ['nmcli', 'device', 'wifi', 'connect', targetSSID, 'ifname', foundInterface]
        if targetPassword != None:
            cliParams += ['password', targetPassword]

        cli = subprocess.Popen(cliParams,
                stdin =subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                bufsize=0)
        cli.stdin.close()

        result = False
        cmdOutput = []
        for line in cli.stdout:
            cmdOutput.append(line)
            if line.find('successfully activated'):
                result = True

        timeCost['connected'] = time.time() - timeCost['connected']
        timeCost['total'] = time.time() - timeCost['total']
        cmdOutput = "\n".join(cmdOutput)
        if result == False:
            cmdOutput = f"stdout:\n{cmdOutput}\n\nstderr:\n{cli.stderr.read()}\n"

        return (result, f"time cost: {timeCost}, cmd: {cmdOutput}")

    def scanToGetAPList(self, targetInterface:str | None = None):
        output = { 'status': False, 'list': [] , 'error': None }

        checkInterface = self.getInterface()
        foundInterface = None
        if targetInterface == None:
            foundInterface = checkInterface['default']
        for iface in checkInterface['list']:
            if targetInterface == iface:
                foundInterface = targetInterface

        if foundInterface == None:
            return output

        cli = subprocess.Popen(['nmcli', 'device', 'wifi', 'list', 'ifname', foundInterface],
                stdin =subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                bufsize=0)
        cli.stdin.close()

        fields = ["IN-USE", "BSSID", "SSID", "MODE", "CHAN", "RATE", "SIGNAL","BARS", "SECURITY"]
        fieldsInfo = {}

        getTitle = False
        for line in cli.stdout:
            if getTitle == False:
                checkPass = True
                for fieldName in fields:
                    beginOffset = line.find(fieldName)
                    if beginOffset > 0 and line[beginOffset - 1] != ' ':
                        beginOffset = line.find(fieldName, beginOffset + 1)
                    if beginOffset == -1:
                        checkPass = False
                        break
                    fieldsInfo[fieldName] = [beginOffset, -1]

                if checkPass:
                    for fieldName in fields:
                        beginOffset = fieldsInfo[fieldName][0]
                        nextOffset = -1
                        for findNextFieldName in fields:
                            if findNextFieldName == fieldName:
                                continue
                            if fieldsInfo[findNextFieldName][0] > beginOffset and (nextOffset == -1 or fieldsInfo[findNextFieldName][0] < nextOffset):
                                nextOffset = fieldsInfo[findNextFieldName][0]
                        if nextOffset != -1:
                            fieldsInfo[fieldName][1] = nextOffset - 1  
                if checkPass:
                    getTitle = True
            else:
                item = {}
                for fieldName in fields:
                    item[fieldName] = line[ fieldsInfo[fieldName][0] : fieldsInfo[fieldName][1] ].strip() if fieldsInfo[fieldName][1] != -1 else line[ fieldsInfo[fieldName][0]: ].strip() 
                #print(item)
                itemOutput = {
                    yy_wifi_helper.WIFIAP.SSID: item['SSID'],
                    yy_wifi_helper.WIFIAP.BSSID: item['BSSID'],
                    yy_wifi_helper.WIFIAP.RSSI: None,
                    yy_wifi_helper.WIFIAP.IBSS: None,
                    yy_wifi_helper.WIFIAP.SECURITY: item['SECURITY'],
                    yy_wifi_helper.WIFIAP.CHANNEL_BAND: None,
                    yy_wifi_helper.WIFIAP.CHANNEL_NUMBER: item['CHAN'],
                    yy_wifi_helper.WIFIAP.CHANNEL_WIDTH: None,
                    yy_wifi_helper.WIFIAP.RAW : item
                }
                output['list'].append(itemOutput)

        output['status'] = len(output['list']) > 0

        return output

    def getInterface(self):
        output = { 'default': None, 'list': [], 'error': None }
        cli = subprocess.Popen(['nmcli', 'device', 'show'],
                stdin =subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                bufsize=0)
        cli.stdin.close()

        item = {}
        for line in cli.stdout:
            keyValue = line.split(":", 2)
            if len(keyValue) == 2:
                key = keyValue[0].strip()
                value = keyValue[1].strip()
                item[key] = value
            else:
                if len(item) > 0 and 'GENERAL.TYPE' in item and item['GENERAL.TYPE'] == 'wifi' and 'GENERAL.DEVICE' in item:
                    if output['default'] == None:
                        output['default'] = item['GENERAL.DEVICE']
                    output['list'].append(item['GENERAL.DEVICE'])
                item = {}
        
        return output

    def scanToGetAPListInJSON(self, targetInterface: str | None = None):
        output = self.scanToGetAPList(targetInterface)
        if output['status'] == True:
            for i in range(len(output['list'])):
                if 'object' in output['list'][i]:
                    del output['list'][i]['object']
        try:
            return json.dumps(output, indent=4)
        except Exception as e:
            return {'error': str(e)}



# -*- encoding: utf-8 -*-

import time
import platform
import json
import logging
import subprocess

# https://pyobjc.readthedocs.io/en/latest/notes/framework-wrappers.html
import objc
import CoreWLAN

from . import yy_wifi_helper
from .macos_wifi_permission_helper import MacOSWiFiPermissionHelper
from .config import WiFiScannerConfig

# https://github.com/ronaldoussoren/pyobjc/tree/master/pyobjc-framework-CoreWLAN
# https://developer.apple.com/documentation/corewlan/cweventdelegate?language=objc
# https://github.com/ronaldoussoren/pyobjc/blob/master/pyobjc-framework-CoreWLAN/PyObjCTest/test_cwwificlient.py
# CoreWLAN.CWEventDelegate

# https://developer.apple.com/documentation/corewlan/cweventdelegate/1512395-linkdidchangeforwifiinterfacewit?language=objc
class YYMacOSCoreWLANHelper(CoreWLAN.NSObject):
    def __init__(self):
        print("in YYMacOSCoreWLANHelper init")
        self.eventHelper = None

    def setCallback_(self, handler, debug:bool = False):
        if handler != None and hasattr(handler, 'eventCallback'):
            self.eventHelper = handler
        self.eventHelperDebug = debug

    def removeCallback(self):
        self.eventHelper = None

    def bssidDidChangeForWiFiInterfaceWithName_(self, interfaceName):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print(f"bssidDidChangeForWiFiInterfaceWithName: {interfaceName}")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"bssidDidChangeForWiFiInterfaceWithName: {interfaceName}",
            })

    def clientConnectionInterrupted(self):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print("clientConnectionInterrupted")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': 'clientConnectionInterrupted',
            })

    def clientConnectionInvalidated(self):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print("clientConnectionInvalidated")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': 'clientConnectionInvalidated',
            })

    def countryCodeDidChangeForWiFiInterfaceWithName_(self, interfaceName):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print(f"countryCodeDidChangeForWiFiInterfaceWithName: {interfaceName}")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"countryCodeDidChangeForWiFiInterfaceWithName: {interfaceName}",
            })

    def linkDidChangeForWiFiInterfaceWithName_(self, interfaceName):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print(f"linkDidChangeForWiFiInterfaceWithName: {interfaceName}")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"linkDidChangeForWiFiInterfaceWithName: {interfaceName}",
            })

    def modeDidChangeForWiFiInterfaceWithName_(self, interfaceName):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print(f"modeDidChangeForWiFiInterfaceWithName: {interfaceName}")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"modeDidChangeForWiFiInterfaceWithName: {interfaceName}",
            })

    def linkQualityDidChangeForWiFiInterfaceWithName_rssi_transmitRate_(self, nm, rs, tr):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print("linkQualityDidChangeForWiFiInterfaceWithName_rssi_transmitRate_")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"linkQualityDidChangeForWiFiInterfaceWithName: {nm}, rssi: {rs}, rate: {tr}",
            })

    def powerStateDidChangeForWiFiInterfaceWithName_(self, interfaceName):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print(f"powerStateDidChangeForWiFiInterfaceWithName: {interfaceName}")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"powerStateDidChangeForWiFiInterfaceWithName: {interfaceName}",
            })

    def ssidDidChangeForWiFiInterfaceWithName_(self, interfaceName):
        if hasattr(self, 'eventHelperDebug') and self.eventHelperDebug:
            print(f"ssidDidChangeForWiFiInterfaceWithName: {interfaceName}")
        if hasattr(self, 'eventHelper') and self.eventHelper != None and callable(self.eventHelper.eventCallback):
            self.eventHelper.eventCallback({
                'raw': f"ssidDidChangeForWiFiInterfaceWithName: {interfaceName}",
            })

class YYMacOSWIFIHelper(yy_wifi_helper.YYOSWIFIHelper):
    def __init__(self):
        self.client = CoreWLAN.CWWiFiClient.alloc().init()
        #print(CoreWLAN.CWWiFiClient.interfaceNames())
        self.systemEventHandler = None
        self._permission_helper = None
        self._config = WiFiScannerConfig()

    def __del__(self):
        self.disableEventHandler()
        del self.client
        self.client = None
        if self.systemEventHandler != None:
            self.systemEventHandler.removeCallback()
        self.systemEventHandler = None

    def _format_error_message(self, error_str):
        """格式化錯誤訊息並提供幫助提示"""
        if "Command" in error_str and "returned non-zero exit status" in error_str:
            return [
                "WiFiScanner.app not found or not properly set up.",
                "Please run 'py-wifi-helper-macos-setup' to set up WiFi scanning permissions.",
                "You can also specify a custom location with:",
                "1. py-wifi-helper-macos-setup --target-path /path/to/your/WiFiScanner.app",
                "2. py-wifi-helper --action scan --scanner-path /path/to/your/WiFiScanner.app"
            ]
        return [error_str]

    @property
    def permission_helper(self):
        if self._permission_helper is None:
            self._permission_helper = MacOSWiFiPermissionHelper(
                app_path=self._config.scanner_app_path
            )
        return self._permission_helper

    def disableEventHandler(self):
        if self.systemEventHandler == None:
            return
        success = self.client.stopMonitoringAllEventsAndReturnError_(None)
        if success:
            print("self.client.stopMonitoringAllEventsAndReturnError_ success")
        else:
            print("self.client.stopMonitoringAllEventsAndReturnError_ error")
        self.client.setDelegate_(None)

    def enableEventHandler(self, handler:yy_wifi_helper.YYWIFIHelper, debug:bool = False ):
        if self.systemEventHandler == None:
            self.systemEventHandler = YYMacOSCoreWLANHelper.alloc().init()
            if handler != None:
                self.systemEventHandler.setCallback_(handler, debug)
        
            # https://developer.apple.com/documentation/corewlan/cweventtype?language=objc
            self.client.setDelegate_(self.systemEventHandler)
            for option in [
                    CoreWLAN.CWEventTypeSSIDDidChange,
                    CoreWLAN.CWEventTypeLinkDidChange,
                    CoreWLAN.CWEventTypeLinkQualityDidChange,
                    CoreWLAN.CWEventTypeScanCacheUpdated,
                    CoreWLAN.CWEventTypeCountryCodeDidChange,
                    CoreWLAN.CWEventTypeBSSIDDidChange,
                    #CoreWLAN.CWEventTypeScanCacheUpdated,
                    #CoreWLAN.CWEventTypeUnknown,
                    #CoreWLAN.CWEventTypeNone,
            ]:
                success = self.client.startMonitoringEventWithType_error_(option, None)
                if success:
                    if debug:
                        print(f"self.client.startMonitoringEventWithType_error_ success: {option}")
    
    def getConnectedAPSSID(self, targetInterface: str | None = None) -> (bool, str | None, str | None):
        output = None
        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                return (False, output, f"interface not found: {targetInterface}")
        except Exception as e:
            return (False, output, f"find interface error: {targetInterface}, {str(e)}")

        targetRealInterface = None
        try:
            # https://developer.apple.com/documentation/corewlan/cwwificlient/1512328-interface
            targetRealInterface = self.client.interfaceWithName_(targetInterface)
        except Exception as e:
            targetRealInterface = None
            return (False, output, f"client.interface(withName): {str(e)}")

        if targetRealInterface == None:
            return (False, output, f"interface is empty")

        try:
            currentNetworkSSID = targetRealInterface.ssid()
            output = currentNetworkSSID
        except Exception as e:
            return (False, output, f"get ssid error: {str(e)}")

        return (True, output)

    def disconnect(self, targetInterface:str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> (bool, str):
        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                return (False, f"interface not found: {targetInterface}")
        except Exception as e:
            return (False, f"find interface error: {targetInterface}, {str(e)}")

        targetRealInterface = None
        try:
            # https://developer.apple.com/documentation/corewlan/cwwificlient/1512328-interface
            targetRealInterface = self.client.interfaceWithName_(targetInterface)
        except Exception as e:
            targetRealInterface = None
            return (False, f"client.interface(withName): {str(e)}")

        if targetRealInterface == None:
            return (False, f"interface is empty")

        try:
            targetRealInterface.disassociate()
        except Exception as e:
            return (False, f"disassociate error: {str(e)}")

        if asyncMode == False:
                waitCount = asyncWaitTimeout + 0.5
                currentConnectedSSID = None
                while waitCount > 0.5:
                    queryStatus, currentConnectedSSID = self.getConnectedAPSSID(targetInterface)
                    if queryStatus and currentConnectedSSID == None:
                        break
                    time.sleep(0.1)
                    waitCount -= 0.1
                if currentConnectedSSID != None:
                    return (False, 'disassociate timeout')

        return (True, None)

    def connectToAP(self, targetSSID:str , targetPassword: str | None = None, targetSecurity: yy_wifi_helper.YYWIFISecurityMode | None = None, findSSIDBeforeConnect:bool = False, targetInterface: str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> (bool, str):
        timeCost = {
            'total': 0.0,
            'scan': 0.0,
            'connect': 0.0,
            'connected': 0.0,
        }
        timeCost['total'] = time.time()
        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                return (False, f"interface not found: {targetInterface}")
        except Exception as e:
            return (False, f"find interface error: {targetInterface}, {str(e)}")

        targetRealInterface = None
        try:
            # https://developer.apple.com/documentation/corewlan/cwwificlient/1512328-interface
            targetRealInterface = self.client.interfaceWithName_(targetInterface)
        except Exception as e:
            targetRealInterface = None
            return (False, f"client.interface(withName): {str(e)}")

        if targetRealInterface == None:
            return (False, f"interface is empty")

        queryCurrentConnectedSSID, queryLog = self.getConnectedAPSSID(targetInterface)
        if queryCurrentConnectedSSID == False:
            return (False, f"getConnectedAP error: {queryLog}")
        if queryLog != None:
            if queryLog == targetSSID:
                return (True, f"errorMessage: Already connected to the specified SSID")

        result = False
        errorMessage = None
        if findSSIDBeforeConnect:
            timeCost['scan'] = time.time()
            # https://developer.apple.com/documentation/corewlan/cwinterface/1426436-scanfornetworkswithssid?language=objc
            errorLog = None
            networks, error = targetRealInterface.scanForNetworksWithSSID_error_(targetSSID.encode("utf-8"), errorLog)
            timeCost['scan'] = time.time() - timeCost['scan']
            if error:
                return (False, f"targetRealInterface.scanForNetworksWithName_error_: {targetSSID}, error: {str(error)}, errorLog: {str(errorLog)}")
            if len(networks) == 0:
                return (False, f"targetRealInterface.scanForNetworksWithName_error_: no networks (find '{targetSSID}')")

            targetNetwork = None
            for n in networks:
                targetNetwork = n

            if targetNetwork == None:
                return (False, f"targetNetwork is empty")

            timeCost['connect'] = time.time()
            try:
                # https://developer.apple.com/documentation/corewlan/cwinterface/1426455-associatetonetwork?language=objc
                success = targetRealInterface.associateToNetwork_password_error_(targetNetwork, targetPassword, errorLog)
                if success:
                    result = True
                else:
                    errorMessage = f"connect error with log: {str(errorLog)}"
            except Exception as e:
                result = False

            timeCost['connect'] = time.time() - timeCost['connect']
        else:
            return (False, 'TODO - macOS Not Support( findSSIDBeforeConnect = False )')
            networkProfile = CoreWLAN.CWMutableNetworkProfile.alloc().init()
            networkProfile.setSsidData_(targetSSID.encode('utf-8'))
            if targetPassword != None:
                # https://developer.apple.com/documentation/corewlan/cwsecurity?language=objc
                networkProfile.setSecurity_(CoreWLAN.kCWSecurityWPA2Personal)
            #config.phrase = targetPassword

            networkConfig = CoreWLAN.CWMutableConfiguration.alloc().init()
            networkConfig.setNetworkProfiles_([networkProfile])

            success: Unknown = targetRealInterface.commitConfiguration_authorization_error_(networkConfig, None, objc.NULL)
            #if success:
            #    print("Done Sucess")

        if result:
            if asyncMode == False:
                timeCost['connected'] = time.time()
                waitCount = asyncWaitTimeout + 0.5
                currentConnectedSSID = None
                while waitCount > 0.5:
                    queryStatus, queryLog = self.getConnectedAPSSID(targetInterface)
                    if queryStatus and queryLog != None and queryLog == targetSSID:
                        currentConnectedSSID = queryLog
                        break
                    time.sleep(0.1)
                    waitCount -= 0.1
                result = currentConnectedSSID == targetSSID
                timeCost['connected'] = time.time() - timeCost['connected']

        timeCost['total'] = time.time() - timeCost['total']
        errorMessage = f"time cost: {timeCost}" if errorMessage == None else f"{errorMessage}, time cost: {timeCost}"
        return (result, errorMessage)

    def scanToGetAPList(self, targetInterface:str | None = None):
        output = { 'status': False, 'list': [] , 'error': None }
        
        # 確保 WiFiScanner.app 存在
        if not self.permission_helper.ensure_scanner_app():
            if output['error'] is None:
                output['error'] = []
            error_msg = [
                "WiFiScanner.app not found.",
                "Please run one of the following commands:",
                "1. py-wifi-helper-macos-setup",
                "2. py-wifi-helper-macos-setup --target-path /your/preferred/location/WiFiScanner.app"
            ]
            output['error'].extend(error_msg)
            return output

        # 使用 wifiscan CLI 工具
        try:
            cli_path = self._permission_helper.cli_path
            result = subprocess.run(
                [str(cli_path), '--json'],
                capture_output=True,
                text=True,
                check=True
            )
            
            try:
                data = json.loads(result.stdout)
                if 'networks' in data and data['networks']:
                    output['status'] = True
                    for network in data['networks']:
                        network_info = {
                            yy_wifi_helper.WIFIAP.SSID: network['ssid'],
                            yy_wifi_helper.WIFIAP.RSSI: network['signal_strength'],
                            yy_wifi_helper.WIFIAP.SECURITY: network['security'],
                            yy_wifi_helper.WIFIAP.CHANNEL_NUMBER: network['channel'],
                        }
                        output['list'].append(network_info)
                else:
                    if output['error'] is None:
                        output['error'] = []
                    output['error'].extend([
                        "Location Services permission required for WiFi scanning.",
                        "Please run 'py-wifi-helper-macos-setup' to set up permissions.",
                        "Or specify a custom scanner location with:",
                        "py-wifi-helper --action scan --scanner-path /path/to/WiFiScanner.app"
                    ])
            except json.JSONDecodeError as e:
                if output['error'] is None:
                    output['error'] = []
                output['error'].append(f"Failed to parse scan results: {str(e)}")
                
        except subprocess.CalledProcessError as e:
            if output['error'] is None:
                output['error'] = []
            output['error'].append(f"Scan failed: {str(e)}")
            output['error'].extend(self._format_error_message(f"Scan failed: {str(e)}"))
        except Exception as e:
            if output['error'] is None:
                output['error'] = []
            output['error'].extend(self._format_error_message(f"Unexpected error: {str(e)}"))
        
        return output

    def scanToGetAPListOld(self, targetInterface:str | None = None):
        output = { 'status': False, 'list': [] , 'error': None }
        target = None
        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                errorMessage = f"interface not found: {targetInterface}, current: {currentInterface['list']}"
                output['error'] = [errorMessage]
                return output
        except Exception as e:
            errorMessage = f"interface error: {targetInterface}"
            if output['error'] is None:
                output['error'] = [errorMessage]
            else:
                output['error'].append(errorMessage) 
            return output

        try:
            # https://developer.apple.com/documentation/corewlan/cwwificlient/1512328-interface
            target = self.client.interfaceWithName_(targetInterface)
        except Exception as e:
            target = None
            errorMessage = f"client.interface(withName): {str(e)}"
            if output['error'] is None:
                output['error'] = [errorMessage]
            else:
                output['error'].append(errorMessage) 

        if target == None:
            return output

        checkScanForNetworksStatus = True
        try:
            ssid, err = target.scanForNetworksWithName_error_(None, None)
            if err == None:
                output['status'] = True
                for item in ssid:
                    try:
                        itemOutput = {
                            yy_wifi_helper.WIFIAP.SSID: item.ssid(),
                            yy_wifi_helper.WIFIAP.BSSID: item.bssid(),
                            yy_wifi_helper.WIFIAP.RSSI: item.rssiValue(),
                            yy_wifi_helper.WIFIAP.IBSS: item.ibss(),
                            yy_wifi_helper.WIFIAP.SECURITY: None,
                            yy_wifi_helper.WIFIAP.CHANNEL_BAND: None,
                            yy_wifi_helper.WIFIAP.CHANNEL_NUMBER: None,
                            yy_wifi_helper.WIFIAP.CHANNEL_WIDTH: None,
                        }

                        try:
                            itemOutput[yy_wifi_helper.WIFIAP.SECURITY] = item.security()
                        except Exception as e:
                            checkScanForNetworksStatus = False
                            errorMessage = f"item.security(): {str(e)}"
                            if output['error'] is None:
                                output['error'] = [errorMessage]
                            else:
                                output['error'].append(errorMessage) 

                        try:
                            itemOutput[yy_wifi_helper.WIFIAP.CHANNEL_BAND] = item.wlanChannel().channelBand()
                            itemOutput[yy_wifi_helper.WIFIAP.CHANNEL_NUMBER] = item.wlanChannel().channelNumber()
                            itemOutput[yy_wifi_helper.WIFIAP.CHANNEL_WIDTH] = item.wlanChannel().channelWidth()
                        except Exception as e:
                            checkScanForNetworksStatus = False
                            errorMessage = f"item.wlanChannel()->channelBand(),channelNumber(),channelWidth(): {str(e)}"
                            if output['error'] is None:
                                output['error'] = [errorMessage]
                            else:
                                output['error'].append(errorMessage) 

                        itemOutput['object'] = item
                        output['list'].append(itemOutput)
                    except Exception as e:
                        checkScanForNetworksStatus = False
                        errorMessage = f"item->ssid(), bssid(), rssiValue(), ibss(): {str(e)}"
                        if output['error'] is None:
                            output['error'] = [errorMessage]
                        else:
                            output['error'].append(errorMessage) 
            else:
                checkScanForNetworksStatus = False
                errorMessage = f"scanForNetworksWithName_error_: {str(err)}"
                if output['error'] is None:
                    output['error'] = [errorMessage]
                else:
                    output['error'].append(errorMessage) 
        except Exception as e:
            checkScanForNetworksStatus = False
            errorMessage = f"item->ssid(), bssid(), rssiValue(), ibss(): {str(e)}"
            if output['error'] is None:
                output['error'] = [errorMessage]
            else:
                output['error'].append(errorMessage) 

        if checkScanForNetworksStatus == False:
            try:
                if not self.permission_helper.check_permissions():
                    self.permission_helper.ensure_scanner_app()
                    self.permission_helper.request_permissions()
                scan_result = self.permission_helper.scan_wifi()
                if scan_result and 'networks' in scan_result:
                    output['status'] = True
                    for network in scan_result['networks']:
                        network_info = {
                            yy_wifi_helper.WIFIAP.SSID: network['ssid'],
                            yy_wifi_helper.WIFIAP.RSSI: network['signal_strength'],
                            yy_wifi_helper.WIFIAP.SECURITY: network['security'],
                            yy_wifi_helper.WIFIAP.CHANNEL_NUMBER: network['channel'],
                        }
                        output['list'].append(network_info)
            except Exception as e:
                if output['error'] is None:
                    output['error'] = []
                output['error'].append(f"WiFiScanner.app scan failed: {str(e)}")
    
        return output

    def getInterface(self):
        output = { 'default': None, 'list': [], 'error': None }

        # https://developer.apple.com/documentation/corewlan/cwwificlient
        try:
            output['default'] = self.client.interface().interfaceName()
        except Exception as e:
            output['error'] = [e] if output['error'] is None else output['error'].append(e)

        try:
            for item in self.client.interfaces():
                try:
                    output['list'].append(item.interfaceName())
                except Exception as e:
                    output['error'] = [e] if output['error'] is None else output['error'].append(e)
        except Exception as e:
            output['error'] = [e] if output['error'] is None else output['error'].append(e)
        
        return output

    def scanToGetAPListInJSON(self, targetInterface: str | None = None):
        output = self.scanToGetAPList(targetInterface)
        if output['status'] == True:
            for i in range(len(output['list'])):
                del output['list'][i]['object']
        try:
            return json.dumps(output, indent=4)
        except Exception as e:
            return {'error': str(e)}



import os
import json
from pathlib import Path

class WiFiScannerConfig:
    def __init__(self):
        self.config_file = os.path.expanduser('~/.py_wifi_helper.json')
        self._load_config()

    def _load_config(self):
        try:
            with open(self.config_file, 'r') as f:
                self._config = json.load(f)
        except FileNotFoundError:
            self._config = {}
            self._save_config()

    def _save_config(self):
        os.makedirs(os.path.dirname(self.config_file), exist_ok=True)
        with open(self.config_file, 'w') as f:
            json.dump(self._config, f, indent=2)

    @property
    def scanner_app_path(self):
        return self._config.get('scanner_app_path')

    @scanner_app_path.setter
    def scanner_app_path(self, path):
        if path:
            path = os.path.expanduser(path)
            if not path.endswith('.app'):
                path = os.path.join(path, 'WiFiScanner.app')
        self._config['scanner_app_path'] = path
        self._save_config()



import os
import subprocess
import json
from pathlib import Path
import time
import logging
import sys
import traceback

class MacOSWiFiPermissionHelper:
    def __init__(self, app_name="WiFiScanner", debug=False, app_path=None):
        """Initialize the WiFi permission helper."""
        self.debug = debug
        
        # 設定 app 路徑
        if app_path:
            self.app_path = Path(os.path.expanduser(app_path))
            if self.app_path.name.endswith('.app'):
                self.app_name = self.app_path.stem  # 從路徑獲取名稱
            else:
                self.app_name = app_name
                self.app_path = self.app_path / f"{app_name}.app"
        else:
            self.app_name = app_name
            self.app_path = Path(os.getcwd()) / f"{app_name}.app"
            
        # 確保父目錄存在
        self.app_path.parent.mkdir(parents=True, exist_ok=True)
            
        self.cli_path = self.app_path / "Contents/MacOS/wifiscan"
        
        # Setup logger
        self.logger = logging.getLogger(__name__)
        if not self.logger.handlers:
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter('%(message)s'))
            self.logger.addHandler(handler)
            self.logger.setLevel(logging.DEBUG if debug else logging.INFO)
    
    def ensure_scanner_app(self, force=False):
        """Ensure WiFiScanner.app exists and is properly signed."""
        try:
            if self.debug:
                self.logger.debug(f"Expected app path: {self.app_path}")
                self.logger.debug(f"Current working directory: {os.getcwd()}")
            
            if force and self.app_path.exists():
                if self.debug:
                    self.logger.debug(f"Removing existing app at: {self.app_path}")
                import shutil
                shutil.rmtree(self.app_path)
            
            if not self.app_path.exists():
                if self.debug:
                    self.logger.debug("Generating new app...")
                    
                from .macos_app_generator import generate_app
                success, message = generate_app(
                    self.app_name,
                    quiet=not self.debug,
                    target_dir=str(self.app_path),  # 直接傳遞完整的 .app 路徑
                    debug=self.debug
                )
                
                if not success:
                    self.logger.error(f"App generation failed: {message}")
                    return False
                
                if not self.app_path.exists():
                    self.logger.error(f"App was not created at: {self.app_path}")
                    if self.debug:
                        self.logger.debug(f"Directory writable: {os.access(self.app_path.parent, os.W_OK)}")
                        self.logger.debug(f"Directory listing: {os.listdir(self.app_path.parent)}")
                    return False
            
            if self.debug:
                self.logger.debug(f"App exists: {self.app_path.exists()}")
                self.logger.debug(f"CLI exists: {self.cli_path.exists()}")
            
            return self.app_path.exists() and self.cli_path.exists()
            
        except Exception as e:
            self.logger.error(f"Failed to generate scanner app: {e}")
            if self.debug:
                self.logger.debug(traceback.format_exc())
            return False

    def check_permissions(self):
        """Check if we have required permissions by attempting a scan."""
        if not self.cli_path.exists():
            if self.debug:
                self.logger.debug(f"CLI tool not found at: {self.cli_path}")
            return False
            
        try:
            result = subprocess.run(
                [str(self.cli_path), '--json'],
                capture_output=True,
                text=True,
                check=True,
                timeout=10
            )
            
            try:
                data = json.loads(result.stdout)
                has_networks = 'networks' in data and len(data['networks']) > 0
                if self.debug:
                    self.logger.debug(f"Scan result: {has_networks}, networks count: {len(data.get('networks', []))}")
                return has_networks
            except json.JSONDecodeError:
                if self.debug:
                    self.logger.debug(f"Failed to parse JSON response: {result.stdout}")
                return False
                
        except subprocess.CalledProcessError as e:
            if self.debug:
                self.logger.debug(f"Command failed: {e}")
            return False
        except Exception as e:
            if self.debug:
                self.logger.debug(f"Permission check failed: {e}")
            return False
    
    def request_permissions(self):
        """Request WiFi scanning permissions by opening the app."""
        if not self.app_path.exists():
            self.logger.error("WiFiScanner.app not found")
            if self.debug:
                self.logger.debug(f"Expected path: {self.app_path}")
            return False
            
        try:
            self.logger.info("\nOpening WiFiScanner.app to request permissions...")
            result = subprocess.run(['open', str(self.app_path)], 
                                  capture_output=True, 
                                  text=True,
                                  check=True)
            
            if self.debug:
                self.logger.debug(f"Open command output: {result.stdout}")
            
            self.logger.info("\nPlease check your notification center for permission requests.")
            self.logger.info("You might need to:")
            self.logger.info("1. Allow Location Services access when prompted")
            self.logger.info("2. Go to System Settings > Privacy & Security > Location Services")
            self.logger.info("3. Find and enable WiFiScanner in the list")
            
            self.logger.info("\nWaiting for permission response...")
            for i in range(6):  # 30 seconds total
                if self.check_permissions():
                    self.logger.info("\n✓ Permissions successfully granted!")
                    return True
                if i < 5:
                    self.logger.info("Still waiting for permissions... (press Ctrl+C to cancel)")
                time.sleep(5)
            
            self.logger.info("\n× Permission request timed out")
            self.logger.info("If you haven't seen the permission request:")
            self.logger.info("1. Try running 'py-wifi-helper-macos-setup --force'")
            self.logger.info("2. Or manually enable Location Services for WiFiScanner in System Settings")
            return False
            
        except subprocess.CalledProcessError as e:
            self.logger.error(f"\n× Error opening WiFiScanner.app: {e}")
            if self.debug:
                self.logger.debug(f"Command output: {e.output if hasattr(e, 'output') else 'No output'}")
            return False
        except KeyboardInterrupt:
            self.logger.info("\n× Setup cancelled by user")
            self.logger.info("You can run 'py-wifi-helper-macos-setup' again at any time")
            return False
        except Exception as e:
            self.logger.error(f"\n× Unexpected error: {e}")
            if self.debug:
                self.logger.debug(traceback.format_exc())
            return False

    def scan_wifi(self):
        """Scan WiFi networks using the CLI tool."""
        if not self.cli_path.exists():
            self.logger.error("WiFiScanner CLI tool not found")
            return None
            
        try:
            result = subprocess.run(
                [str(self.cli_path), '--json'],
                capture_output=True,
                text=True,
                check=True
            )
            return json.loads(result.stdout)
            
        except subprocess.CalledProcessError as e:
            self.logger.error(f"Scan failed: {e}")
            return None
        except json.JSONDecodeError as e:
            self.logger.error(f"Failed to parse scan results: {e}")
            return None
        except Exception as e:
            self.logger.error(f"Unexpected error during scan: {e}")
            if self.debug:
                self.logger.debug(traceback.format_exc())
            return None



# -*- encoding: utf-8 -*-
import sys
import argparse
import datetime
import time
import json

from py_wifi_helper import __version__
from py_wifi_helper import yy_wifi_helper
from .config import WiFiScannerConfig

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--action", choices=['device', 'scan', 'connect', 'disconnect'], 
                       default='device', help="command action")
    parser.add_argument("--device", type=str, default=None, help="interface")
    parser.add_argument("--ssid", type=str, help="ssid")
    parser.add_argument("--password", type=str, help="password")
    parser.add_argument("--scanner-path", type=str, help="Path to WiFiScanner.app (macOS only)")
    args = parser.parse_args()

    if args.scanner_path and sys.platform == 'darwin':
        config = WiFiScannerConfig()
        config.scanner_app_path = args.scanner_path

    obj = yy_wifi_helper.YYWIFIHelper()

    output = {}
    output['platform'] = obj._platform
    output['version'] = __version__
    output['device'] = obj.getInterface()
    output['device']['select'] = args.device if args.device != None else output['device']['default']
    output['connection'] = { 'default': { 'ssid': None, 'log': None } }

    for deviceId in output['device']['list']:
        queryStatus, queryLog = obj.getConnectedAPSSID(deviceId)
        output['connection'][deviceId] = { 'ssid': None, 'log': None }
        if queryStatus:
            output['connection'][deviceId]['ssid'] = queryLog
        else:
            output['connection'][deviceId]['log'] = queryLog
        if deviceId == output['device']['select']:
            output['connection']['default'] = output['connection'][deviceId]
        
    output['action'] = {
        'name': args.action,
        'status' : False,
        'error': None,
        'log': None,
    }
    if args.action == 'device':
        if output['device']['error'] != None:
            output['action']['error'] = output['device']['error']
        else:
            output['action']['status'] = True
        print(json.dumps(output, indent=4))
        sys.exit(0)

    if args.action == 'scan':
        output['ssid'] = []
        query = obj.getAPList(output['device']['select'])
        if 'status' in query and query['status']:
            output['action']['status'] = True
            for item in query['list']:
                if item['ssid'] == None:
                    continue
                output['ssid'].append(item['ssid'])
            output['ssid'].sort()
        elif 'error' in query:
            output['action']['error'] = query['error']
        print(json.dumps(output, indent=4))
        sys.exit(0)

    if args.action == 'disconnect':
        queryStatus, queryError = obj.disconnect(output['device']['select'])
        if queryStatus:
            output['action']['status'] = True
        else:
            output['action']['error'] = queryError
        print(json.dumps(output, indent=4))
        sys.exit(0)

    if args.action == 'connect':
        if args.ssid == None:
            output['action']['error'] = 'target ssid is empty'
        else:
            queryStatus, querylog = obj.connectToAP(args.ssid, args.password, targetInterface=output['device']['select'], findSSIDBeforeConnect=True)
            if queryStatus:
                output['action']['status'] = True
                output['action']['log'] = querylog
            else:
                output['action']['error'] = querylog
        print(json.dumps(output, indent=4))
        sys.exit(0)

    output['action']['error'] = 'Not Found'
    print(json.dumps(output, indent=4))
    return

if __name__ == '__main__':
    main()



# -*- encoding: utf-8 -*-

import os
import platform
import json
from enum import Enum
from typing import Dict, List, Optional, Any, Tuple

class WIFIAP_SECURITY(Enum):
    pass

class WIFIAP(str, Enum):
    SSID = 'ssid'
    BSSID = 'bssid'
    RSSI = 'rssi'
    IBSS = 'ibbs'
    SECURITY = 'security'
    CHANNEL_BAND = 'channel.band'
    CHANNEL_NUMBER = 'channel.number'
    CHANNEL_WIDTH = 'channel.width'
    RAW = 'raw'

class YYWIFISecurityMode(str, Enum):
    SecurityNone = 'SecurityNone'
    SecurityWEP = 'SecurityWEP'
    SecurityWPAPersonal = 'SecurityWPAPersonal'
    SecurityWPAPersonalMixed = 'SecurityWPAPersonalMixed'
    SecurityWPA2Personal = 'SecurityWPA2Personal'
    SecurityPersonal = 'SecurityPersonal'
    SecurityDynamicWEP = 'SecurityDynamicWEP'
    SecurityWPAEnterprise = 'SecurityWPAEnterprise'
    SecurityWPAEnterpriseMixed = 'SecurityWPAEnterpriseMixed'
    SecurityWPA2Enterprise = 'SecurityWPA2Enterprise'
    SecurityEnterprise = 'SecurityEnterprise'
    SecurityWPA3Personal = 'SecurityWPA3Personal'
    SecurityWPA3Enterprise = 'SecurityWPA3Enterprise'
    SecurityWPA3Transition = 'SecurityWPA3Transition'
    SecurityUnknown = 'SecurityUnknown'

class YYOSWIFIHelper:
    def disableEventHandler(self):
        pass

    def enableEventHandler(self, handler: None, debug:bool = False ):
        pass

    def getConnectedAPSSID(self, targetInterface: str | None = None) -> Tuple[bool, Optional[str], Optional[str]]:
        pass

    def getInterface(self) -> Dict[str, Any]:
        return {'default': None, 'list': [], 'error': None}

    def disconnect(self, targetInterface:str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> Tuple[bool, str]:
        pass

    def connectToAP(self, targetSSID:str , targetPassword: str | None = None, targetSecurity: YYWIFISecurityMode | None = None, findSSIDBeforeConnect:bool = False, targetInterface: str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> Tuple[bool, str]:
        pass

    def scanToGetAPList(self, targetInterface:str | None = None) -> Dict[str, Any]:
        pass

    def scanToGetAPListInJSON(self, targetInterface: str | None = None) -> str:
        pass

class YYWIFIHelper:
    def __init__(self):
        # 初始化基本屬性
        self._platform = platform.system().lower()
        self._supportPlatform = ['darwin']
        self._platformInfo = {}
        self._helper = None
        self._support = False
        self._missing_dependencies = []
        self._init_error = None
        self.eventHandler = None
        
        # Windows 平台支援
        if self._platform == 'windows':
            try:
                print("Checking required modules...")
                import comtypes
                import pywifi
                print("Initializing WiFi...")
                test_wifi = pywifi.PyWiFi()
                self._supportPlatform.append('windows')
                self._support = True
            except ImportError as e:
                self._missing_dependencies.append(('dependency', str(e)))
                print(f"[ERROR] Missing dependency: {str(e)}")
            except Exception as e:
                self._init_error = str(e)
                print(f"[ERROR] Initialization error: {str(e)}")
        
        # Linux/Ubuntu 平台支援
        elif self._platform == 'linux':
            if os.path.isfile('/etc/lsb-release'):
                for line in open('/etc/lsb-release').read().split('\n'):
                    keyValue = line.split('=')
                    if len(keyValue) == 2:
                        self._platformInfo[keyValue[0].strip().lower()] = keyValue[1].strip().lower()

            if 'distrib_id' in self._platformInfo:
                if self._platformInfo['distrib_id'] == 'ubuntu' and 'distrib_release' in self._platformInfo:
                    try:
                        if float(self._platformInfo['distrib_release']) >= 22.04:
                            from shutil import which
                            cliCheckPass = True
                            cliList = ['nmcli']
                            for cli in cliList:
                                if which(cli) is None:
                                    print(f"[INFO] cli not found: {cli}")
                                    cliCheckPass = False
    
                            if cliCheckPass:
                                self._supportPlatform.append('ubuntu')
                                self._platform = 'ubuntu'
                                self._support = True
                    except Exception as e:
                        self._init_error = str(e)
                        print(f"[ERROR] {str(e)}")
        
        # macOS 平台支援
        elif self._platform == 'darwin':
            try:
                import objc
                import CoreWLAN
                self._support = True
            except ImportError as e:
                self._missing_dependencies.append(('CoreWLAN', str(e)))
                print(f"[ERROR] {str(e)}")

        # 初始化對應平台的 helper
        if self._support:
            try:
                if self._platform == 'darwin':
                    from . import my_macos_helper
                    self._helper = my_macos_helper.YYMacOSWIFIHelper()
                elif self._platform == 'ubuntu':
                    from . import my_ubuntu_helper
                    self._helper = my_ubuntu_helper.YYUbuntuWIFIHelper()
                elif self._platform == 'windows':
                    from . import my_windows_helper
                    print("Initializing Windows WiFi Helper...")
                    self._helper = my_windows_helper.YYWindowsWIFIHelper()
            except Exception as e:
                self._support = False
                self._init_error = f"Helper initialization failed: {str(e)}"
                print(f"[ERROR] {self._init_error}")

    def eventCallback(self, info: Dict | None = None):
        if self.eventHandler and callable(self.eventHandler):
            self.eventHandler(info)

    def enableEventHandler(self, handlerFunc: None):
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return
        self.eventHandler = handlerFunc
        if self._helper:
            self._helper.enableEventHandler(self)

    def disableEventHandler(self):
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return
        if self._helper:
            self._helper.disableEventHandler()
        self.eventHandler = None

    def getInterface(self) -> Dict[str, Any]:
        if not self._support:
            error_msg = []
            if self._init_error:
                error_msg.append(self._init_error)
            if self._missing_dependencies:
                for dep, err in self._missing_dependencies:
                    error_msg.append(f"Missing {dep}: {err}")
            if not error_msg:
                error_msg.append(f"Platform {self._platform} not supported")
                
            return {
                'default': None,
                'list': [],
                'error': error_msg
            }
        
        if not self._helper:
            return {
                'default': None,
                'list': [],
                'error': ['Helper not initialized']
            }
            
        return self._helper.getInterface()

    def getAPList(self, name: Optional[str] = None) -> Dict[str, Any]:
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return {'status': False, 'list': [], 'error': ['Platform not supported']}
        if not self._helper:
            return {'status': False, 'list': [], 'error': ['Helper not initialized']}
        return self._helper.scanToGetAPList(name)

    def getAPListInJSON(self, name: Optional[str] = None) -> str:
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return json.dumps({'status': False, 'list': [], 'error': ['Platform not supported']})
        if not self._helper:
            return json.dumps({'status': False, 'list': [], 'error': ['Helper not initialized']})
        return self._helper.scanToGetAPListInJSON(name)

    def getConnectedAPSSID(self, targetInterface: Optional[str] = None) -> Tuple[bool, Optional[str]]:
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return (False, None, 'Platform not supported')
        if not self._helper:
            return (False, None, 'Helper not initialized')
        return self._helper.getConnectedAPSSID(targetInterface)

    def disconnect(self, targetInterface: Optional[str] = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> Tuple[bool, str]:
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return (False, 'Platform not supported')
        if not self._helper:
            return (False, 'Helper not initialized')
        return self._helper.disconnect(targetInterface, asyncMode, asyncWaitTimeout)

    def connectToAP(self, targetSSID: str, targetPassword: Optional[str] = None, targetSecurity: Optional[str] = None, findSSIDBeforeConnect: bool = False, targetInterface: Optional[str] = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> Tuple[bool, str]:
        if not self._support:
            print(f"Platform not supported: {self._platform}")
            return (False, 'Platform not supported')
        if not self._helper:
            return (False, 'Helper not initialized')
        return self._helper.connectToAP(targetSSID, targetPassword, targetSecurity, findSSIDBeforeConnect, targetInterface, asyncMode, asyncWaitTimeout)



import os
import sys
import argparse
import logging
from .macos_wifi_permission_helper import MacOSWiFiPermissionHelper
from .config import WiFiScannerConfig

def setup_logging(debug=False):
    level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(
        level=level,
        format='%(message)s'
    )

def handle_macos_permission(force=False, debug=False, target_path=None):
    setup_logging(debug)
    
    config = WiFiScannerConfig()
    
    if target_path is None:
        default_path = config.scanner_app_path or os.getcwd()
        user_input = input(f"Enter path for WiFiScanner.app [default: {default_path}]: ").strip()
        target_path = user_input if user_input else default_path

    # 確保路徑是絕對路徑
    target_path = os.path.expanduser(target_path)
    config.scanner_app_path = target_path
    
    helper = MacOSWiFiPermissionHelper(debug=debug, app_path=target_path)
    
    logging.info("Checking macOS WiFi permissions...")
    if not force and helper.check_permissions():
        logging.info("✓ WiFi scanning permissions are already granted")
        return 0

    logging.info("× WiFi scanning permissions not found")
    logging.info(f"\nGenerating WiFiScanner.app at: {target_path}")
    
    if not helper.ensure_scanner_app(force):
        logging.error("\n× Failed to generate WiFiScanner.app")
        return 1

    logging.info("\nInitiating permission request...")
    if not helper.request_permissions():
        logging.error("\n× Failed to complete permission setup")
        logging.error("Please try running 'py-wifi-helper-macos-setup' again")
        return 1

    if not helper.check_permissions():
        logging.error("\n× Permission verification failed")
        logging.error("If you haven't seen the permission request:")
        logging.error("1. Try running 'py-wifi-helper-macos-setup' again")
        logging.error("2. Or manually enable Location Services for WiFiScanner in System Settings")
        return 1

    logging.info("\n✓ Setup completed successfully!")
    logging.info("You can now use 'py-wifi-helper --action scan' to scan for networks")
    return 0

def main():
    parser = argparse.ArgumentParser(description='macOS WiFi Permission Setup')
    parser.add_argument('--force', action='store_true', 
                      help='Force regenerate WiFiScanner.app')
    parser.add_argument('--debug', action='store_true',
                      help='Enable debug logging')
    parser.add_argument('--target-path', 
                      help='Target path for WiFiScanner.app')
    args = parser.parse_args()
    
    if sys.platform != 'darwin':
        print("This command is only for macOS systems")
        return 1
        
    try:
        return handle_macos_permission(args.force, args.debug, args.target_path)
    except KeyboardInterrupt:
        print("\n× Setup cancelled by user")
        return 1
    except Exception as e:
        print(f"\n× Setup failed with error: {e}")
        if args.debug:
            import traceback
            traceback.print_exc()
        return 1

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3
import os
import plistlib
import shutil
from pathlib import Path
import subprocess
import sys
import logging
import traceback

def check_dependencies(quiet=False):
    """Check and install required packages."""
    required_packages = [
        'pyobjc-framework-CoreWLAN',
        'pyobjc-framework-CoreLocation',
        'pyobjc-framework-Cocoa'
    ]
    
    if not quiet:
        print("Checking and installing required packages...")
        
    try:
        for package in required_packages:
            subprocess.check_call(
                [sys.executable, '-m', 'pip', 'install', package],
                stdout=subprocess.PIPE if quiet else None,
                stderr=subprocess.PIPE if quiet else None
            )
            if not quiet:
                print(f"✓ {package} installed/verified")
        return True
    except subprocess.CalledProcessError as e:
        if not quiet:
            print(f"Error installing {package}: {e}")
        return False

def sign_app(app_path, quiet=False):
    """Sign the application to enable permission requests."""
    try:
        # Remove extended attributes
        subprocess.run(
            ['xattr', '-cr', app_path],
            check=True,
            stdout=subprocess.PIPE if quiet else None,
            stderr=subprocess.PIPE if quiet else None
        )
        
        # Sign the application
        subprocess.run([
            'codesign',
            '--force',
            '--deep',
            '--sign',
            '-',
            app_path
        ], check=True,
            stdout=subprocess.PIPE if quiet else None,
            stderr=subprocess.PIPE if quiet else None
        )
        
        if not quiet:
            print("✓ Application signed successfully")
        return True
    except subprocess.CalledProcessError as e:
        if not quiet:
            print(f"Error signing application: {e}")
        return False

def generate_app(app_name="WiFiScanner", quiet=False, target_dir=None, debug=False):
    """Generate the WiFiScanner.app"""
    try:
        if not check_dependencies(quiet):
            return False, "Failed to install required dependencies"

        # 處理目標目錄和應用名稱
        if target_dir is None:
            target_dir = os.getcwd()
            final_app_path = os.path.join(target_dir, f"{app_name}.app")
        else:
            # 展開用戶目錄並獲取絕對路徑
            target_dir = os.path.expanduser(target_dir)
            target_dir = os.path.abspath(target_dir)
            
            # 如果目標路徑以 .app 結尾
            if target_dir.endswith('.app'):
                final_app_path = target_dir
                app_name = os.path.basename(target_dir)[:-4]  # 移除 .app
                target_dir = os.path.dirname(target_dir)
            else:
                final_app_path = os.path.join(target_dir, f"{app_name}.app")

        if debug:
            print(f"Target directory: {target_dir}")
            print(f"App name: {app_name}")
            print(f"Final app path: {final_app_path}")
            print(f"Current working directory: {os.getcwd()}")

        # 確保目標目錄存在
        try:
            os.makedirs(target_dir, exist_ok=True)
        except Exception as e:
            return False, f"Failed to create target directory {target_dir}: {str(e)}"

        # 檢查目錄權限
        if not os.access(target_dir, os.W_OK):
            return False, f"Target directory is not writable: {target_dir}"

        # 設定應用路徑
        app_path = final_app_path
        contents_path = os.path.join(app_path, "Contents")
        macos_path = os.path.join(contents_path, "MacOS")
        resources_path = os.path.join(contents_path, "Resources")

        if debug:
            print(f"Creating app at: {app_path}")

        # 如果已存在則移除
        if os.path.exists(app_path):
            if debug:
                print(f"Removing existing app at: {app_path}")
            shutil.rmtree(app_path)

        # 創建目錄結構
        for path in [macos_path, resources_path]:
            os.makedirs(path, exist_ok=True)
            if debug:
                print(f"Created directory: {path}")

        # Generate shared WiFi scanner module
        wifi_scanner_path = os.path.join(macos_path, "wifi_scanner.py")
        with open(wifi_scanner_path, 'w') as f:
            f.write('''#!/usr/bin/env python3
import CoreWLAN
import CoreLocation
import objc

def get_security_mode_str(network):
    """Convert security mode to human readable string"""
    try:
        mode = network.securityMode()
        if mode == CoreWLAN.kCWSecurityNone:
            return "None"
        elif mode == CoreWLAN.kCWSecurityWEP:
            return "WEP"
        elif mode == CoreWLAN.kCWSecurityWPAPersonal:
            return "WPA Personal"
        elif mode == CoreWLAN.kCWSecurityWPAPersonalMixed:
            return "WPA Personal Mixed"
        elif mode == CoreWLAN.kCWSecurityWPA2Personal:
            return "WPA2 Personal"
        elif mode == CoreWLAN.kCWSecurityPersonal:
            return "Personal"
        elif mode == CoreWLAN.kCWSecurityDynamicWEP:
            return "Dynamic WEP"
        elif mode == CoreWLAN.kCWSecurityWPAEnterprise:
            return "WPA Enterprise"
        elif mode == CoreWLAN.kCWSecurityWPAEnterpriseMixed:
            return "WPA Enterprise Mixed"
        elif mode == CoreWLAN.kCWSecurityWPA2Enterprise:
            return "WPA2 Enterprise"
        elif mode == CoreWLAN.kCWSecurityEnterprise:
            return "Enterprise"
        else:
            return f"Unknown ({mode})"
    except Exception:
        return "Unknown"

def scan_wifi():
    """Scan WiFi networks and return results"""
    try:
        wifi = CoreWLAN.CWWiFiClient.sharedWiFiClient()
        interface = wifi.interface()
        
        if not interface:
            return None, "No WiFi interface found"
        
        networks, error = interface.scanForNetworksWithName_includeHidden_error_(None, True, None)
        if error:
            return None, str(error)
        
        if not networks:
            return None, "No networks found"
        
        return networks, None
        
    except Exception as e:
        return None, str(e)
''')

        # Generate CLI tool
        cli_script_path = os.path.join(macos_path, "wifiscan")
        with open(cli_script_path, 'w') as f:
            f.write('''#!/usr/bin/env python3
import os
import sys
import json
import argparse

# Add current directory to Python path
current_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, current_dir)

from wifi_scanner import scan_wifi, get_security_mode_str

def format_scan_results(networks, args):
    """Format scan results according to args"""
    if not networks:
        return []
        
    results = []
    for network in networks:
        ssid = network.ssid()
        if ssid:
            net_info = {
                "ssid": ssid,
                "signal_strength": network.rssiValue(),
                "security": get_security_mode_str(network),
                "channel": network.wlanChannel().channelNumber()
            }
            results.append(net_info)
    
    if args.sort:
        results.sort(key=lambda x: x["signal_strength"], reverse=True)
    
    if args.min_strength:
        results = [n for n in results if n["signal_strength"] >= args.min_strength]
    
    if args.ssid:
        results = [n for n in results if args.ssid.lower() in n["ssid"].lower()]
    
    if args.limit:
        results = results[:args.limit]
    
    return results

def print_results(networks, args):
    """Print results in the specified format"""
    if args.json:
        print(json.dumps({"networks": networks}, indent=2))
        return
    
    if not networks:
        print("No networks found matching criteria")
        return
    
    print(f"Found {len(networks)} networks:")
    for net in networks:
        print(f"\\nSSID: {net['ssid']}")
        print(f"  Signal Strength: {net['signal_strength']}dB")
        print(f"  Security: {net['security']}")
        print(f"  Channel: {net['channel']}")

def main():
    os.chdir(current_dir)
    
    parser = argparse.ArgumentParser(description="WiFi Network Scanner")
    parser.add_argument("--json", action="store_true", help="Output in JSON format")
    parser.add_argument("--sort", action="store_true", help="Sort by signal strength")
    parser.add_argument("--min-strength", type=int, help="Minimum signal strength in dB")
    parser.add_argument("--ssid", help="Filter by SSID (case insensitive)")
    parser.add_argument("--limit", type=int, help="Limit the number of results")
    args = parser.parse_args()

    networks, error = scan_wifi()
    if error:
        print(f"Error: {error}")
        return 1
        
    results = format_scan_results(networks, args)
    print_results(results, args)
    return 0

if __name__ == "__main__":
    sys.exit(main())
''')
        os.chmod(cli_script_path, 0o755)

        # Generate main application script
        main_script_path = os.path.join(macos_path, "main.py")
        if debug:
            print(f"Generating main script at: {main_script_path}")
            
        with open(main_script_path, 'w', encoding='utf-8') as f:
            f.write('''#!/usr/bin/env python3
import os
import sys
import time
import AppKit
import Foundation
from wifi_scanner import scan_wifi, get_security_mode_str

def main():
    # Initialize notification center
    notification_center = Foundation.NSUserNotificationCenter.defaultUserNotificationCenter()

    # Create notification
    notification = AppKit.NSUserNotification.alloc().init()
    notification.setTitle_("WiFi Scanner")
    notification.setInformativeText_("Started WiFi scanning...")
    
    # Show notification
    notification_center.deliverNotification_(notification)
    
    try:
        networks, error = scan_wifi()
        
        if networks:
            result = ["Found WiFi networks:"]
            for network in networks:
                ssid = network.ssid()
                if ssid:
                    result.append(f"SSID: {ssid}, Signal: {network.rssiValue()}dB")
                    security = get_security_mode_str(network)
                    result.append(f"  Security: {security}")
                    result.append("  Channel: {}".format(network.wlanChannel().channelNumber()))
                    result.append("")
            
            # Create result notification
            notification = AppKit.NSUserNotification.alloc().init()
            notification.setTitle_("WiFi Scan Complete")
            notification.setInformativeText_("\\n".join(result[:5]) + "\\n...")
            notification_center.deliverNotification_(notification)
            
            # Write full results to file
            result_path = os.path.join(os.path.dirname(__file__), "scan_result.txt")
            with open(result_path, "w", encoding='utf-8') as f:
                f.write("\\n".join(result))
        else:
            notification = AppKit.NSUserNotification.alloc().init()
            notification.setTitle_("WiFi Scan Failed")
            notification.setInformativeText_(error or "Unknown error")
            notification_center.deliverNotification_(notification)
    
    except Exception as e:
        notification = AppKit.NSUserNotification.alloc().init()
        notification.setTitle_("WiFi Scan Error")
        notification.setInformativeText_(str(e))
        notification_center.deliverNotification_(notification)

if __name__ == "__main__":
    main()
    time.sleep(2)
''')
        os.chmod(main_script_path, 0o755)

        if debug:
            print(f"Generated main script: {os.path.exists(main_script_path)}")

        # Generate launcher script
        launcher_script_path = os.path.join(macos_path, "run")
        with open(launcher_script_path, 'w') as f:
            f.write('''#!/usr/bin/env python3
import os
import sys
import subprocess

def main():
    script_dir = os.path.dirname(os.path.abspath(__file__))
    main_script = os.path.join(script_dir, "main.py")
    
    env = os.environ.copy()
    python_path = subprocess.check_output(['python3', '-c', 'import site; print(site.getsitepackages()[0])'], text=True).strip()
    env['PYTHONPATH'] = f"{{env.get('PYTHONPATH', '')}}:{{python_path}}"
    
    os.chdir(script_dir)
    os.execve(sys.executable, [sys.executable, main_script], env)

if __name__ == "__main__":
    main()
''')
        os.chmod(launcher_script_path, 0o755)

        # Generate Info.plist
        info_plist = {
            'CFBundleDisplayName': app_name,
            'CFBundleExecutable': 'run',
            'CFBundleIconFile': 'AppIcon',
            'CFBundleIdentifier': f'com.example.{app_name.lower()}',
            'CFBundleName': app_name,
            'CFBundlePackageType': 'APPL',
            'CFBundleShortVersionString': '1.0',
            'LSMinimumSystemVersion': '10.10',
            'NSHighResolutionCapable': True,
            'NSSupportsAutomaticGraphicsSwitching': True,
            'LSBackgroundOnly': True,
            'NSLocationUsageDescription': 'This app needs location access to scan for WiFi networks',
            'NSLocationWhenInUseUsageDescription': 'This app needs location access to scan for WiFi networks',
        }
        
        plist_path = os.path.join(contents_path, 'Info.plist')
        with open(plist_path, 'wb') as f:
            plistlib.dump(info_plist, f)

        # Sign the application
        if not sign_app(app_path, quiet):
            return False, "Failed to sign the application"

        # 最終驗證
        paths_to_verify = [
            (app_path, "App"),
            (contents_path, "Contents"),
            (macos_path, "MacOS"),
            (resources_path, "Resources"),
            (cli_script_path, "CLI tool"),
            (plist_path, "Info.plist")
        ]
        
        for path, name in paths_to_verify:
            if not os.path.exists(path):
                return False, f"Failed to verify {name} at: {path}"
            if debug:
                print(f"✓ {name} exists at: {path}")

        return True, f"Application generated successfully at {app_path}"

    except Exception as e:
        tb = traceback.format_exc() if debug else ""
        error_msg = f"Failed to generate app: {str(e)}\n{tb}"
        return False, error_msg

if __name__ == "__main__":
    success, message = generate_app(debug=True)
    if not success:
        print(f"Error: {message}")
        sys.exit(1)
    sys.exit(0)



# -*- encoding: utf-8 -*-

import time
import platform
import json
import pywifi
from pywifi import const

from . import yy_wifi_helper

class YYWindowsWIFIHelper(yy_wifi_helper.YYOSWIFIHelper):
    def __init__(self):
        self.wifi = pywifi.PyWiFi()
        self.eventHandler = None

    def __del__(self):
        self.disableEventHandler()
        self.wifi = None

    def disableEventHandler(self):
        self.eventHandler = None

    def enableEventHandler(self, handler:yy_wifi_helper.YYWIFIHelper, debug:bool = False):
        self.eventHandler = handler
    
    def getConnectedAPSSID(self, targetInterface: str | None = None) -> (bool, str | None, str | None):
        """獲取指定介面當前連接的 WiFi SSID"""
        output = None
        
        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                return (False, output, f"interface not found: {targetInterface}")
        except Exception as e:
            return (False, output, f"find interface error: {targetInterface}, {str(e)}")

        targetRealInterface = None
        try:
            for iface in self.wifi.interfaces():
                if iface.name() == targetInterface:
                    targetRealInterface = iface
                    break
        except Exception as e:
            return (False, output, f"get interface error: {str(e)}")

        if targetRealInterface is None:
            return (False, output, f"interface is empty")

        try:
            if targetRealInterface.status() == const.IFACE_CONNECTED:
                try:
                    # 獲取當前連接的配置
                    profile = targetRealInterface.network_profiles()[0]
                    output = profile.ssid
                except IndexError:
                    return (False, output, "No network profile found")
            return (True, output)
        except Exception as e:
            return (False, output, f"get ssid error: {str(e)}")

    def disconnect(self, targetInterface:str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> (bool, str):
        """斷開指定介面的 WiFi 連接"""
        timeCost = {
            'total': time.time(),
        }

        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                return (False, f"interface not found: {targetInterface}")
        except Exception as e:
            return (False, f"find interface error: {targetInterface}, {str(e)}")

        targetRealInterface = None
        try:
            for iface in self.wifi.interfaces():
                if iface.name() == targetInterface:
                    targetRealInterface = iface
                    break
        except Exception as e:
            return (False, f"get interface error: {str(e)}")

        if targetRealInterface is None:
            return (False, f"interface is empty")

        try:
            targetRealInterface.disconnect()
            
            if not asyncMode:
                waitCount = asyncWaitTimeout
                while waitCount > 0:
                    if targetRealInterface.status() != const.IFACE_CONNECTED:
                        break
                    time.sleep(0.1)
                    waitCount -= 0.1
                if targetRealInterface.status() == const.IFACE_CONNECTED:
                    return (False, "disconnect timeout")
            
            timeCost['total'] = time.time() - timeCost['total']
            return (True, f"time cost: {timeCost}")
            
        except Exception as e:
            return (False, f"disconnect error: {str(e)}")

    def connectToAP(self, targetSSID:str, targetPassword: str | None = None, targetSecurity: yy_wifi_helper.YYWIFISecurityMode | None = None, findSSIDBeforeConnect:bool = False, targetInterface: str | None = None, asyncMode: bool = False, asyncWaitTimeout: int = 15) -> (bool, str):
        """連接到指定的 WiFi 網路"""
        timeCost = {
            'total': time.time(),
            'scan': 0.0,
            'connect': 0.0,
            'connected': 0.0,
        }

        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                return (False, f"interface not found: {targetInterface}")
        except Exception as e:
            return (False, f"find interface error: {targetInterface}, {str(e)}")

        targetRealInterface = None
        try:
            for iface in self.wifi.interfaces():
                if iface.name() == targetInterface:
                    targetRealInterface = iface
                    break
        except Exception as e:
            return (False, f"get interface error: {str(e)}")

        if targetRealInterface is None:
            return (False, f"interface is empty")

        # 檢查當前連接
        try:
            currentStatus, currentSSID = self.getConnectedAPSSID(targetInterface)
            if currentStatus and currentSSID == targetSSID:
                timeCost['total'] = time.time() - timeCost['total']
                return (True, f"Already connected to {targetSSID}, time cost: {timeCost}")
        except Exception:
            pass

        try:
            if findSSIDBeforeConnect:
                timeCost['scan'] = time.time()
                targetRealInterface.scan()
                time.sleep(2)
                scan_results = targetRealInterface.scan_results()
                timeCost['scan'] = time.time() - timeCost['scan']
                
                found_network = False
                for result in scan_results:
                    if result.ssid == targetSSID:
                        found_network = True
                        break
                
                if not found_network:
                    return (False, f"Network {targetSSID} not found")

            # 創建網路配置
            profile = pywifi.Profile()
            profile.ssid = targetSSID
            profile.auth = const.AUTH_ALG_OPEN

            if targetPassword:
                profile.akm = [const.AKM_TYPE_WPA2PSK]
                profile.cipher = const.CIPHER_TYPE_CCMP
                profile.key = targetPassword
            else:
                profile.akm = [const.AKM_TYPE_NONE]
                profile.cipher = const.CIPHER_TYPE_NONE

            timeCost['connect'] = time.time()
            
            # 移除現有配置並添加新配置
            targetRealInterface.remove_all_network_profiles()
            tmp_profile = targetRealInterface.add_network_profile(profile)
            
            # 嘗試連接
            targetRealInterface.connect(tmp_profile)
            
            timeCost['connect'] = time.time() - timeCost['connect']

            if not asyncMode:
                timeCost['connected'] = time.time()
                waitCount = asyncWaitTimeout
                while waitCount > 0:
                    if targetRealInterface.status() == const.IFACE_CONNECTED:
                        break
                    time.sleep(0.1)
                    waitCount -= 0.1
                    
                if targetRealInterface.status() != const.IFACE_CONNECTED:
                    return (False, f"Connection timeout")
                    
                timeCost['connected'] = time.time() - timeCost['connected']

            timeCost['total'] = time.time() - timeCost['total']
            return (True, f"time cost: {timeCost}")
            
        except Exception as e:
            return (False, f"connect error: {str(e)}")

    def scanToGetAPList(self, targetInterface:str | None = None):
        """掃描可用的 WiFi 網路"""
        output = {'status': False, 'list': [], 'error': None}

        try:
            currentInterface = self.getInterface()
            if targetInterface is None:
                targetInterface = currentInterface['default']
            elif targetInterface not in currentInterface['list']:
                output['error'] = [f"interface not found: {targetInterface}, current: {currentInterface['list']}"]
                return output
        except Exception as e:
            output['error'] = [f"interface error: {str(e)}"]
            return output

        targetRealInterface = None
        try:
            for iface in self.wifi.interfaces():
                if iface.name() == targetInterface:
                    targetRealInterface = iface
                    break
        except Exception as e:
            output['error'] = [f"get interface error: {str(e)}"]
            return output

        if targetRealInterface is None:
            output['error'] = [f"interface is empty"]
            return output

        try:
            targetRealInterface.scan()
            time.sleep(2)
            results = targetRealInterface.scan_results()
            
            for result in results:
                try:
                    itemOutput = {
                        yy_wifi_helper.WIFIAP.SSID: result.ssid,
                        yy_wifi_helper.WIFIAP.BSSID: result.bssid,
                        yy_wifi_helper.WIFIAP.RSSI: result.signal,
                        yy_wifi_helper.WIFIAP.IBSS: None,
                        yy_wifi_helper.WIFIAP.SECURITY: str(result.akm[0]) if result.akm else None,
                        yy_wifi_helper.WIFIAP.CHANNEL_BAND: None,
                        yy_wifi_helper.WIFIAP.CHANNEL_NUMBER: result.freq,
                        yy_wifi_helper.WIFIAP.CHANNEL_WIDTH: None,
                        yy_wifi_helper.WIFIAP.RAW: {
                            'signal': result.signal,
                            'quality': result.quality,
                            'frequency': result.freq,
                            'akm': [str(x) for x in result.akm] if result.akm else [],
                            'cipher': str(result.cipher) if result.cipher else None,
                        }
                    }
                    output['list'].append(itemOutput)
                except Exception as e:
                    if output['error'] is None:
                        output['error'] = []
                    output['error'].append(f"parse network error: {str(e)}")
                    
            output['status'] = len(output['list']) > 0
            
        except Exception as e:
            if output['error'] is None:
                output['error'] = []
            output['error'].append(f"scan error: {str(e)}")

        return output

    def getInterface(self):
        """獲取所有可用的無線網卡介面"""
        output = {'default': None, 'list': [], 'error': None}
        
        try:
            interfaces = self.wifi.interfaces()
            
            for interface in interfaces:
                try:
                    name = interface.name()
                    if name not in output['list']:
                        output['list'].append(name)
                        if output['default'] is None:
                            output['default'] = name
                except Exception as e:
                    if output['error'] is None:
                        output['error'] = []
                    output['error'].append(f"get interface name error: {str(e)}")
                    
        except Exception as e:
            if output['error'] is None:
                output['error'] = []
            output['error'].append(f"get interfaces error: {str(e)}")
            
        return output

    def scanToGetAPListInJSON(self, targetInterface: str | None = None):
        """以 JSON 格式返回掃描結果"""
        output = self.scanToGetAPList(targetInterface)
        try:
            return json.dumps(output, indent=4)
        except Exception as e:
            return {'error': str(e)}
